package com.gic.store.web.exception;

import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.store.constant.StoreGroupErrorEnum;
import com.gic.store.exception.StoreGroupException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.util.List;

/**
 * 全局异常处理类
 *
 * @author hua
 */
@ControllerAdvice
public class GlobalExceptionHandler {
    private static Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    @ResponseBody
    @ExceptionHandler(Exception.class)
    public RestResponse controllerException(HttpServletResponse response, Exception ex) {
        logger.error("err", ex);
        RestResponse failureResponse = getRestResponse(StoreGroupErrorEnum.ERROR.getCode(), StoreGroupErrorEnum.ERROR.getMessage());
        StringBuilder sb = new StringBuilder();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try (PrintWriter printWriter = new PrintWriter(baos)) {
            ex.printStackTrace(printWriter);
        }
        try {
            sb.append(baos.toString());
        } catch (Exception ignored) {
        }
        if (sb.length() == 0) {
            sb.append(ex.getMessage());
        }
        // 输出详细错误信息，便于调试
        failureResponse.setDetailError(sb.toString());
        return failureResponse;
    }

    /**
     * 参数校验异常统一处理
     * @param e
     * @return
     */
    @ResponseBody
    @ExceptionHandler(BindException.class)
    public RestResponse customException(BindException e) {
        List<FieldError> fieldErrors = e.getFieldErrors();
        StringBuilder errorMessage = new StringBuilder();
        fieldErrors.forEach(fieldError -> {
            errorMessage.append(fieldError.getField())
                    .append(":")
                    .append(fieldError.getDefaultMessage())
                    .append(",");
        });
        return getRestResponse(StoreGroupErrorEnum.ERROR.getCode(), errorMessage.toString());
    }

    /**
     * 自定义异常统一处理
     * @param e
     * @return
     */
    @ResponseBody
    @ExceptionHandler(StoreGroupException.class)
    public RestResponse customException(StoreGroupException e) {
        return getRestResponse(e.getErrorCode(), e.getMessage());
    }


    private RestResponse getRestResponse(String errorCode, String message) {
        return RestResponse.failure(errorCode, message);
    }

}
