package com.gic.plug.web.strategy.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.plug.web.strategy.ScreenBackStrategy;
import com.gic.plug.web.utils.ComputeBackUtil;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.StoreFieldDateEnum;
import com.gic.store.constant.StoreFieldTypeEnum;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.service.StoreFieldApiService;
import com.gic.store.service.StoreFieldSelectApiService;
import com.gic.store.utils.StoreRedisKeyUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class StoreCustomBackImpl implements ScreenBackStrategy {
    @Autowired
    private StoreFieldApiService storeFieldApiService;
    @Autowired
    private StoreFieldSelectApiService storeFieldSelectApiService;
    private Map<Integer, String> methodMap = new HashMap<>();

    @PostConstruct
    public void init() {
        methodMap.put(StoreFieldDateEnum.YMDHMS.getCode(), "yyyyMMddHHmmssBack");
        methodMap.put(StoreFieldDateEnum.YMD.getCode(), "yyyyMMddBack");
        methodMap.put(StoreFieldDateEnum.YM.getCode(), "yyyyMM");
        methodMap.put(StoreFieldDateEnum.MD.getCode(), "mmddBack");
        methodMap.put(StoreFieldDateEnum.Hms.getCode(), "hhmmssBack");
    }

    @Override
    public String screenBack(String compute, String value, String key, Integer enterpriseId) {
        if (StringUtils.isNotBlank(value)) {
            Object cache = RedisUtil.getCache(StoreRedisKeyUtils.getStoreFieldKeyC(enterpriseId, key));
            if (cache != null) {
                ServiceResponse<StoreFieldDTO> storeField = this.storeFieldApiService.getStoreField(Integer.valueOf(cache.toString()));
                if (storeField.isSuccess()) {
                    if (storeField.getResult() != null) {
                        if (storeField.getResult().getStoreFieldType() == StoreFieldTypeEnum.DATE.getCode()) {
                            return this.dateBack(compute, value, storeField.getResult());
                        } else {
                            return this.selectBack(compute, value, enterpriseId, storeField.getResult());
                        }
                    }
                }
            }
        }
        return null;
    }

    private String dateBack(String compute, String value, StoreFieldDTO storeFieldDTO) {
        StringBuilder stringBuilder = new StringBuilder(storeFieldDTO.getStoreFieldName()).append(" ");
        stringBuilder.append(ComputeBackUtil.getComputeBack(compute)).append(" ");
        JSONObject jsonObject = JSON.parseObject(storeFieldDTO.getStoreFieldDetail());
        int dateType = jsonObject.getIntValue("dateType");
        String methodName = this.methodMap.get(dateType);
        if (StringUtils.isNotBlank(methodName)) {
            try {
                try {
                    Method method = this.getClass().getMethod(methodName, Class.forName(value.getClass().getCanonicalName()));
                    try {
                        String message = (String) method.invoke(this, value);
                        return stringBuilder.append(message).toString();
                    } catch (IllegalAccessException e) {
                        e.printStackTrace();
                    } catch (InvocationTargetException e) {
                        e.printStackTrace();
                    }
                } catch (ClassNotFoundException e) {
                    e.printStackTrace();
                }
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    private String selectBack(String compute, String value, Integer enterpriseId, StoreFieldDTO storeFieldDTO) {
        StringBuilder stringBuilder = new StringBuilder(storeFieldDTO.getStoreFieldName()).append(" ");
        stringBuilder.append(ComputeBackUtil.getComputeBack(compute)).append(" ");
        List<Integer> list = new ArrayList<>();
        String[] valueArr = value.split(" ");
        for (String s : valueArr) {
            list.add(Integer.valueOf(s));
        }
        ServiceResponse<List<StoreFieldSelectDTO>> listServiceResponse = this.storeFieldSelectApiService.listStoreFieldSelectByIds(enterpriseId, list);
        if (listServiceResponse.isSuccess()) {
            List<StoreFieldSelectDTO> selectDTOS = listServiceResponse.getResult();
            if (CollectionUtils.isNotEmpty(selectDTOS)) {
                for (StoreFieldSelectDTO dto : selectDTOS) {
                    stringBuilder.append("【").append(dto.getStoreFieldSelectName()).append("】").append(" ");
                }
            }
        }
        return stringBuilder.toString();
    }

    public String yyyyMMddHHmmssBack(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat simpleDateFormat1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return this.dateBack(value, simpleDateFormat, simpleDateFormat1);
    }

    public String yyyyMMddBack(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMdd");
        SimpleDateFormat simpleDateFormat1 = new SimpleDateFormat("yyyy-MM-dd");
        return this.dateBack(value, simpleDateFormat, simpleDateFormat1);
    }

    public String yyyyMM(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMM");
        SimpleDateFormat simpleDateFormat1 = new SimpleDateFormat("yyyy-MM");
        return this.dateBack(value, simpleDateFormat, simpleDateFormat1);
    }

    public String mmddBack(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("MMdd");
        SimpleDateFormat simpleDateFormat1 = new SimpleDateFormat("MM-dd");
        return this.dateBack(value, simpleDateFormat, simpleDateFormat1);
    }

    public String hhmmssBack(String value) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HHmmss");
        SimpleDateFormat simpleDateFormat1 = new SimpleDateFormat("HH:mm:ss");
        return this.dateBack(value, simpleDateFormat, simpleDateFormat1);
    }

    private String dateBack(String value, SimpleDateFormat simpleDateFormat, SimpleDateFormat simpleDateFormat1) throws ParseException {
        String[] valueArr = value.split(" ");
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("【").append(simpleDateFormat1.format(simpleDateFormat.parse(valueArr[0]))).append("】").append(" ");
        if (valueArr.length > 1) {
            stringBuilder.append("至");
            stringBuilder.append("【").append(simpleDateFormat1.format(simpleDateFormat.parse(valueArr[1]))).append("】").append(" ");
        }
        return stringBuilder.toString();
    }
}
